/***********************************************************
GMM.PRG

This is a GAUSS program, written by
Bruce E. Hansen
Department of Economics
University of Wisconsin
bhansen@ssc.wisc.edu
http:/www.ssc.wisc.edu/~bhansen


***********************************************************/

trm = .02;            @ Percentile of high-income households deleted  @

/***********************************************************/
output file = gmm.out reset;
/***********************************************************/

smp=1; do while smp<=2;              @ Set to 1 if "urban" or 2 if "rural"  @

let rname =
"Region1"
"Region2"
"Region3"
"Region4"
"Region5"
"Region6"
"Region7"
"Region8"
"Region9"
"Region10"
"Region11"
"Region12"
"Region13";
if smp==1;
    sample="Urban";
    rname="Region0"|rname;
    knot=21840;
elseif smp==2;
    sample="Rural";
    knot=10793;
endif;

dfile = "" $+ sample $+ ".dat";
vn = getname(dfile);
open f1=^dfile;
dat=readr(f1,20000);
f1=close(f1);

abroad=dat[.,1];
domestic=dat[.,2];
inkind=dat[.,3];
gifts=dat[.,4];
totinc=dat[.,5];
retinc=dat[.,10];
yname = "Transfers";

y = abroad+domestic+inkind-gifts;
i = totinc+gifts-abroad-domestic-inkind-retinc;

dat=delif(dat~i~y,(i.<0));         @ delete observations with negative income @
i=sortc(dat[.,28],1);
n = floor(rows(dat)*(1-trm));
dat=delif(dat,(dat[.,28].>i[n]));

retinc=dat[.,10];
i=dat[.,28];
y=dat[.,29];
dat= dat[.,1:27];

dat[.,23]=dat[.,23]-dat[.,20]-dat[.,21]-dat[.,22];
vn[23]="Adults";
xx=(retinc.>0)~retinc~dat[.,12:17];
xn="Has Retr"|vn[10 12:17];

region=dat[.,27];
r=unique(region,1);
rr=rows(r);
xreg=(region.==(r'));
mx = ones(n,1)*(meanc(xx)');

n=rows(dat);
x1=xx~dat[.,18:26]~ones(n,1);		@regressors for reduced form@
x2=xx~dat[.,18:26]~xreg;			@regressors for main equation@
xn=xn|vn[18:26]|rname;			@names of regressors@

@ Reduced form for endogenous income @
rs=xreg[.,1];
ir=selif(i,rs);
xr=selif(x1,rs);
ihat=xr*(ir/xr);
j=2; do while j<=rr;
    rs=xreg[.,j];
    ir=selif(i,rs);
    xr=selif(x1,rs);
    ih=xr*(ir/xr);
    ihat=ihat|ih;
j=j+1;endo;
z=ihat~(ihat.^2)~x2;		@instruments@


"************************************************************";
"";"";
sh = sample $+ " households";
$sh;
print ("Dependent Variable:     " $+ yname);
"Trimmed Top Quantile of Income " trm;
"Observations                      " n;
"__________________________________________________";


/*************************************************/
@ Linear Model @
x=i~x2;
w=invpd(moment(z,0));
zx=z'x;
xxi=invpd(zx'w*zx);
beta=xxi*(zx'w*(z'y));
e = y - x*beta;
w=invpd(moment(z.*e,0));
xxi=invpd(zx'w*zx);
beta=xxi*(zx'w*(z'y));
e = y - x*beta;
v=invpd(zx'invpd(moment(z.*e,0))*zx);
se=sqrt(diag(v));
j=(e'z)*w*(z'e);
dof=cols(z)-cols(x);
ff = "#*.*lG";
xname = "Income"|xn;
"Linear Model";"";
pr = printfm("Variable"~"Estimate"~"St Error",0~0~0,ff~16~8);"";
"       ____________________________________________________";
pr = printfm(xname~beta~se,0~1~1,ff~16~8);"";
if dof>0;
"J-Statistic, dof, pvalue " j~dof~cdfchic(j,dof);
endif;
"__________________________________________________";
"";"";"";

@ Linear Spline @
ic = i - knot;
i1 = (i.<=knot);
i2 = 1-i1;
dx=(ic.*i1)~(ic.*i2);
x = dx~x2;
zx=z'x;
xxi=invpd(zx'w*zx);
beta=xxi*(zx'w*(z'y));
e = y - x*beta;
v=invpd(zx'invpd(moment(z.*e,0))*zx);
se=sqrt(diag(v));
j = (e'z)*w*(z'e);
dof=cols(z)-cols(x);

xname = "Slope_1"|"Slope_2"|xn;
"Linear Spline";"";
pr = printfm("Variable"~"Estimate"~"St Error",0~0~0,ff~16~8);"";
"       ____________________________________________________";
pr = printfm(xname~beta~se,0~1~1,ff~16~8);"";
if dof>0;
"J-Statistic, dof, pvalue " j~dof~cdfchic(j,dof);
endif;
"__________________________________________________";
"";"";"";

mx=meanc(x2);
tr = dx*beta[1:2]+mx'beta[3:rows(beta)];
gdat=i~tr;
gdat=sortc(gdat,1);
graphset;
_plwidth=2;
_pdate="";
fonts("complex simgrma");
sh2 = sample $+ " households";
title(sh2);
ylabel("Transfers (Pesos)");
xlabel("Total Income (Pesos)");
_pxsci = 6; _pysci = 6;
if smp==1;
 xtics(0,200000,50000,5); @300000@
 ytics(0000,20000,5000,5); @0 to 20000@
elseif smp==2;
 xtics(0,80000,20000,4);
 ytics(0,10000,2000,4);
endif;
xy(gdat[.,1],gdat[.,2]);

smp = smp+1; endo;
output off;

/*************************************************/

proc (0) = gr_tr(i,tr);
local sh2,pn,zp;
graphset;
_plwidth=2;
_pdate="";
fonts("complex simgrma");
sh2 = sample $+ " households";
title(sh2);
ylabel("Transfers (Pesos)");
xlabel("Total Income (Pesos)");
_pxsci = 6; _pysci = 6;
if smp==1;
 xtics(0,200000,50000,5); @300000@
 ytics(0000,20000,5000,5); @0 to 20000@
elseif smp==2;
 xtics(0,80000,20000,4);
 ytics(0,10000,2000,4);
endif;
xy(i,tr);
endp;

/*************************************************/
